/**
* The contents of this file are subject to the Mozilla Public 
* License Version 1.1 (the "License"); you may not use this 
* file except in compliance with the License. You may obtain a
* copy of the License at http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an 
* "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express 
* or implied. See the License for the specific language 
* governing rights and limitations under the License.
*
* The Original Code is this file as it was released on
* Sept 18, 2005.
*
* The Initial Developer of the Original Code is Terry Yuen. 
* Portions created by Terry Yuen are Copyright (C) 2005.
* All Rights Reserved.
**/

function YMailNotifierService() {
  this.initService();
}

YMailNotifierService.prototype = {
  state: 0,
  count: 0,
  timer: null,

  alertMode: 0,
  stringBundle: null,

  initService: function() {
    var obs = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);
    obs.addObserver(this, "ymailnotifier-action", false);
    obs.addObserver(this, "profile-after-change", false);
    obs.addObserver(this, "xpcom-shutdown", false);
  },

  startup: function() {
    this.updateAlerter();
    var delay = this.updateTimer();

    if(delay > 0) {
      this.checkNowDelayed();
    }
  },

  observe: function(subject, topic, data) {
    switch(topic) {
      case "ymailnotifier-action":
        switch(data) {

          case "mail-count":
            if(!this.isLoggedIn()) {
              //notify the UI of the login state
              this.count = 0;
              this.state = 3;
              this.notifyStatus(3, this.getLocalizedString("error.mail.not_logged_in"));
            } else {
              this.notifyStatus(0, "Mail Count@" + this.count);
            }
            break;

          case "check-now":
            this.checkNow(true);
            this.updateTimer();  //reset timer
            break;

          case "update-timer":
            this.updateTimer();
            break;

          case "update-alerter":
            this.updateAlerter();
            break;
        }
        break;

      case "timer-callback":
        if(subject == this.timer)this.checkNow();
        break;

      case "profile-after-change":
        this.startup();
        break;

      case "xpcom-shutdown":
        this.destroy();
        break;
    }
  },

  checkNowDelayed: function() {
    const d = this;

    var t = Components.classes["@mozilla.org/timer;1"]
        .createInstance(Components.interfaces.nsITimer);
    t.initWithCallback(
      {
        notify: function() {
          d.checkNow();
        }
      },
      1000,
      Components.interfaces.nsITimer.TYPE_ONE_SHOT
    );
  },

/* ========== MAIL PROTOCOL FUNCTIONS ========== */

  checkNow: function(forceAlert) {
    //first check if the user is logged in
    if(!this.isLoggedIn()) {
      this.count = 0;
      this.state = 3;
      this.notifyStatus(3, this.getLocalizedString("error.mail.not_logged_in"));
      if(forceAlert)this.alertMail();
      return;
    }

    this.state = 1;
    this.notifyStatus(1, this.getLocalizedString("info.mail.checking"));

    /* 404 Not Found as of Aug 2, 2006:
    var url = "http://api2.my.yahoo.com/2.0/content/widgetcount"; */
    //now connect
    var url = "http://update.toolbar.yahoo.com/slv/v4/not?.rand=" + parseInt(Math.random(1000));

    var req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
        .createInstance(Components.interfaces.nsIXMLHttpRequest);
    req.open("GET", url, true);
    const dis = this;
    req.onerror = function() {
      dis.state = 5;
      dis.notifyStatus(5, dis.getLocalizedString("error.mail.connection"));
    };
    req.onload = function() {
      dis.checkNowCallback(req, forceAlert);
    };
    req.setRequestHeader("Cache-Control", "no-cache");
    req.overrideMimeType("text/plain");

    try {
      req.send(null);

    } catch(e) {
      this.state = 5;
      this.notifyStatus(5, this.getLocalizedString("error.mail.connection"));
      return;
    }
  },

  checkNowCallback: function(req, forceAlert) {
    //read response
    if(req.status == 200) {
      try {
        /*var resp = req.responseXML;
        resp = resp.getElementsByTagName("channel")[0];
        resp = resp.getElementsByTagName("description")[0];
        resp = resp.firstChild.nodeValue;*/
        var resp = req.responseText;
        if(!/\x14Y!/.test(resp)) throw "YMailNotifier received a malformed response from the server: " + resp;
        resp = /\x19\x16[^\x00-\x19]*(\d+)[^\x00-\x19]*\x2e\x18/i.exec(resp);
        resp = (resp == null) ? 0 : resp[1];

        this.state = 2;
        var oldCount = this.count;
        this.count = parseInt(resp);
        this.notifyStatus(
          2, 
          this.count + "@" +
          this.getLocalizedString(
            ((this.count == 0) ? "info.mail.no_new_messages" :
              (this.count == 1) ? "info.mail.one_new_message" :
                "info.mail.several_new_messages"), 
            ((this.count > 0) ? this.count : new Date().toLocaleTimeString().match(/[^: ]+:[^: ]+/i))
          )
        );

        //don't alert the user unnecessarily
        if(this.count > oldCount || forceAlert)this.alertMail();

      } catch(e) {
        this.state = 4;
        this.notifyStatus(4, this.getLocalizedString("error.mail.protocol"));
      }

    } else {
      this.state = 5;
      this.notifyStatus(5, this.getLocalizedString("error.mail.server", req.statusText));
    }
  },

  notifyStatus: function(n, s) {
    Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService)
        .notifyObservers(null, "ymailnotifier-status", n + "@" + s);
  },

  isLoggedIn: function() {
    try {
      //Yahoo! sets 2 cookies when logged in. Just check for them here.
      var url = Components.classes["@mozilla.org/network/standard-url;1"]
          .createInstance(Components.interfaces.nsIURI);
      url.spec = "http://yahoo.com";

      var cookies = Components.classes["@mozilla.org/cookieService;1"]
          .createInstance(Components.interfaces.nsICookieService)
          .getCookieString(url, null);

      var required = {"Y":true, "T":true};

      if(cookies) {
        cookies = cookies.split("; ");

        for(var i=0; i<cookies.length; i++) {
          var s = cookies[i].substring(0, cookies[i].indexOf("="));
          if(required[s])required[s] = false;
        }
      }

      for(var i in required) {
        if(required[i])return false;
      }
      return true;

    } catch(e) {
      dump("YMailNotifier: " + e);
      return false;
    }
  },

/* ========== TIMER FUNCTIONS ========== */

  updateTimer: function() {
    var delay = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch)
        .getIntPref("ymailnotifier.timer");

    if(delay > 0)delay = Math.max(delay, 120000);

    if(!this.timer) {
      this.timer = Components.classes["@mozilla.org/timer;1"]
          .createInstance(Components.interfaces.nsITimer);
    } else {
      this.timer.cancel();
    }

    if(delay > 0) {
      this.timer.init(this, delay, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
    }

    return delay;
  },

/* ========== ALERT FUNCTIONS ========== */

  updateAlerter: function() {
    this.alertMode = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch)
        .getIntPref("ymailnotifier.alert");
    return this.alertMode;
  },

  alertMail: function() {
    if(this.alertMode == 0)return;

    //play sound
    if((this.alertMode & 0x1) && this.count > 0) {
      try {
        Components.classes["@mozilla.org/sound;1"]
            .createInstance(Components.interfaces.nsISound)
            .playSystemSound("_moz_mailbeep");

      } catch(e) {
        dump("YMailNotifier cannot play sound because: " + e);
      }
    }

    //show the slider
    if(this.alertMode & 0x2) {
      try {
        var title = this.getLocalizedString("alertservice.title");
        var body = this.getLocalizedString(
          (this.state == 3) ? "alertservice.not_logged_in" :
           (this.count == 0) ? "alertservice.no_new_messages" : 
            (this.count == 1) ? "alertservice.one_new_message" :
             "alertservice.several_new_messages",
          this.count
        );

        if("@mozilla.org/alerts-service;1" in Components.classes) {
          Components.classes["@mozilla.org/alerts-service;1"]
              .getService(Components.interfaces.nsIAlertsService)
              .showAlertNotification("chrome://ymailnotifier/content/logo.png", title, body, false, "", null);

        } else {
          var check = {value: false};
          Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
              .getService(Components.interfaces.nsIPromptService)
              .alertCheck(null, title, body+"\n", this.getLocalizedString("alertservice.checkmsg"), check);

          if(check.value) {
            this.alertMode &= 0x1;
            Components.classes["@mozilla.org/preferences-service;1"]
                .getService(Components.interfaces.nsIPrefBranch)
                .setIntPref("ymailnotifier.alert", this.alertMode);
          }
        }
      } catch(e) {
        dump("YMailNotifier cannot show alert because: " + e);
      }
    }

    //run external program
    if(this.count > 0) {
      try {
        var pref = Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefBranch);

        if(pref.prefHasUserValue("ymailnotifier.external_app")) {

          var app = Components.classes["@mozilla.org/process/util;1"]
              .createInstance(Components.interfaces.nsIProcess);

          app.init(pref.getComplexValue("ymailnotifier.external_app", Components.interfaces.nsILocalFile));
          app.run(false, [], 0);
        }
      } catch(e) {
      dump("YMailNotifier cannot run external app because: " + e);
      }
    }
  },

  getLocalizedString: function(s, v) {
    if(!this.stringBundle) {
      this.stringBundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
          .getService(Components.interfaces.nsIStringBundleService)
          .createBundle("chrome://ymailnotifier/locale/component.properties");
    }

    if(v) {
      return this.stringBundle.formatStringFromName(s, [String(v)], 1);
    } else {
      return this.stringBundle.GetStringFromName(s);
    }
  },

  destroy: function() {
    if(this.timer) {
      this.timer.cancel();
      this.timer = null;
    }
    this.stringBundle = null;

    var obs = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);
    obs.removeObserver(this, "ymailnotifier-action");
    obs.removeObserver(this, "profile-after-change");
    obs.removeObserver(this, "xpcom-shutdown");
  },

  QueryInterface: function(iid) {
    if(!iid.equals(Components.interfaces.nsIObserver) &&
        !iid.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

/* ========== XPCOM FUNCTIONS ========== */

var Module = {
  myCID: Components.ID("{bbfec13d-8cb3-53f3-c852-999eb2a852ca}"),
  myProgID: "@com.gmail.mocking/ymail_notifier;1",
  myFriendlyName: "YMail Notifier",

  registerSelf: function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(
      this.myCID,
      this.myFriendlyName,
      this.myProgID,
      fileSpec,
      location,
      type
    );

    Components.classes["@mozilla.org/categorymanager;1"]
        .getService(Components.interfaces.nsICategoryManager)
        .addCategoryEntry(
          "app-startup", 
          this.myFriendlyName, 
          "service," + this.myProgID, 
          true, true
    );
  },

  unregisterSelf: function(compMgr, location, loader) {
    Components.classes["@mozilla.org/categorymanager;1"]
        .getService(Components.interfaces.nsICategoryManager)
        .deleteCategoryEntry(
          "app-startup", 
          this.myFriendlyName, 
          true
    );
  },

  getClassObject: function(compMgr, cid, iid) {
    if(!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;

    if(!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    return this.myFactory;
  },

  myFactory: {
    createInstance: function(outer, iid) {
      if(outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

      return (new YMailNotifierService()).QueryInterface(iid);
    }
  },

  canUnload: function(compMgr) {
    return true;
  }
};

function NSGetModule(compMgr, fileSpec) {
  return Module;
}